#include <Adafruit_NeoPixel.h>

const int pixelPin = 6;
const int btnPin = 2;
const int potPin = A0;

const int pixelCount = 8;

byte mode = 0;
bool lastButtonState;

Adafruit_NeoPixel strip(pixelCount, pixelPin, NEO_GRB + NEO_KHZ800);

void setup() {
  pinMode(btnPin, INPUT_PULLUP);
  pinMode(potPin, INPUT);
  lastButtonState = digitalRead(btnPin);
  strip.begin();
  strip.show();
}

void loop() {
  switch(mode) {
    case 0: modePoliceLights(); break;
    case 1: modeRainbow(); break;
    case 2: modeRainbowPot(analogRead(potPin)); break;
    case 3: modePot1(analogRead(potPin)); break;
    case 4: modePot2(analogRead(potPin)); break;
  }
  bool currentButtonState = digitalRead(btnPin);
  if(currentButtonState && !lastButtonState) {
    mode = (mode + 1) % 5;
  }
  lastButtonState = currentButtonState;
}

void modePoliceLights() {
  int period = 160;
  unsigned long progress = millis() % period;
  
  for(int x = 0; x < strip.numPixels(); x++) {
    if(x < strip.numPixels() / 2) {
      strip.setPixelColor(x, 255 * (progress < period/2) , 0, 255 * (progress >= period/2));
    } else {
      strip.setPixelColor(x, 255 * (progress >= period/2) , 0, 255 * (progress < period/2));
    }
  }
  strip.show();
}

void modeRainbow() {
  int period = 5000;
  unsigned long progress = millis() % period;

  int dif = 360 / strip.numPixels();

  for(int x = 0; x < strip.numPixels(); x++) {
    setHue(x, dif * (x+1) + (360.0 * progress / period));
  }

  strip.show();
}

void modeRainbowPot(int potValue) {
  int dif = 360 / strip.numPixels();

  for(int x = 0; x < strip.numPixels(); x++) {
    setHue(x, dif * (x+1) + (360.0 * potValue / 1023.0));
  }

  strip.show();
}

void setHue(int pixel, int hue) {
  hue = hue % 360;
  int r,g,b;
  if(hue < 60) {
    r = 255;
    g = map(hue, 0, 59, 0, 254);
    b = 0;
  } else if(hue < 120) {
    r = map(hue, 60, 119, 254, 0);
    g = 255;
    b = 0;
  } else if(hue <  180) {
    r = 0;
    g = 255;
    b = map(hue, 120, 179, 0, 254);
  } else if(hue < 240) {
    r = 0;
    g = map(hue, 180, 239, 254, 0);
    b = 255;
  } else if(hue < 300) {
    r = map(hue, 240, 299, 0, 254);
    g = 0;
    b = 255;
  } else {
    r = 255;
    g = 0;
    b = map(hue, 300, 359, 254, 0);
  }

  strip.setPixelColor(pixel, r, g, b);
}

void modePot1(int potValue) {
  byte red = (byte) (255 * (potValue / 1024.0));
  byte green = 255 - red;
  
  int progress = floor(strip.numPixels() * (potValue / 1024.0));
  for(int x = 0; x < strip.numPixels(); x++) {
    if(x <= progress) {
      strip.setPixelColor(x, red, green, 0);
    } else {
      strip.setPixelColor(x, 0, 0, 0);
    }
  }
  strip.show();
}

void modePot2(int potValue) {
  int singleSegment = 1024 / strip.numPixels();
  int progress = floor(strip.numPixels() * (potValue / 1024.0));
  
  for(int x = 0; x < progress; x++) {
    strip.setPixelColor(x, 255, 0, 0);
  }

  float intensity = (potValue - (progress * singleSegment * 1.0)) / singleSegment;


  byte red = (byte) (255.0 * intensity);
  byte green = 255 - red;
  
  strip.setPixelColor(progress, red * intensity, green * intensity, 0);

  for(int x = progress+1; x < strip.numPixels(); x++) {
    strip.setPixelColor(x, 0, 0, 0);
  }

  strip.show();
}
